@@ -859,6 +859,7 @@ multinomialFit <- function(grp, ..., method="lr", log=TRUE, .twotier=TRUE) {
 	if (length(list(...)) > 0) {
 		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
 	}
+	out <- list()
 	if (is.null(grp$weightColumn)) {
 		wc <- "freq"
 		grp$data <- compressDataFrame(grp$data, wc)
@@ -870,24 +871,29 @@ multinomialFit <- function(grp, ..., method="lr", log=TRUE, .twotier=TRUE) {
 	} else {
 		sumFree <- sum(grp$free)
 	}
-	stat <- CaiHansen2012(grp, method, .twotier)
+	got <- CaiHansen2012(grp, method, .twotier)
+	stat <- got$stat
 	bins <- prod(sapply(grp$spec, function(s) s$outcomes))
-	out <- list(statistic=stat,
-		    df=bins - sumFree - 1)
+	out <- c(out, list(statistic=stat, df=bins - sumFree - 1))
 	out$pval <- pchisq(stat, out$df, lower.tail=FALSE, log.p=log)
 	out$log <- log
 	out$method <- method
+	out$n <- got$n
+	out$omiited <- grp$omitted
 	class(out) <- "summary.multinomialFit"
 	out
 }
 
 print.summary.multinomialFit <- function(x,...) {
 	cat("Full information multinomial fit test\n")
-	part1 <- paste(x$method, "(", x$df, ") = ", round(x$statistic, 2), sep="")
+	part1 <- paste("n = ", x$n, ", ", x$method, "(", x$df, ") = ", round(x$statistic, 2), sep="")
 	if (x$log) {
 		part2 <- paste("log(p) = ", round(x$pval,2), sep="")
 	} else {
 		part2 <- paste("p = ", round(x$pval,4), sep="")
 	}
 	cat(paste("  ", part1, ", ", part2, "\n", sep=""))
+	if (!is.null(x$omitted)) {
+		cat(paste("omitted: ", paste(x$omitted, collapse=", "), "\n", sep=""))
+	}
 }
