@@ -209,78 +209,6 @@ void ba81NormalQuad::setup(double Qwidth, int Qpoints, double *means,
 	}
 }
 
-void ba81NormalQuad::mapDenseSpace(double piece, const double *where,
-				   const double *whereGram, double *latentDist)
-{
-	const int pmax = primaryDims;
-	int gx = 0;
-	int cx = maxAbilities;
-	for (int d1=0; d1 < pmax; d1++) {
-		double piece_w1 = piece * where[d1];
-		latentDist[d1] += piece_w1;
-		for (int d2=0; d2 <= d1; d2++) {
-			double piece_cov = piece * whereGram[gx];
-			latentDist[cx] += piece_cov;
-			++cx; ++gx;
-		}
-	}
-}
-
-void ba81NormalQuad::mapSpecificSpace(int sgroup, double piece, const double *where,
-				      const double *whereGram, double *latentDist)
-{
-	const int pmax = primaryDims;
-
-	int sdim = pmax + sgroup;
-	double piece_w1 = piece * where[pmax];
-	latentDist[sdim] += piece_w1;
-
-	double piece_var = piece * whereGram[triangleLoc0(pmax)];
-	int to = maxAbilities + triangleLoc0(sdim);
-	latentDist[to] += piece_var;
-}
-
-void ba81NormalQuad::EAP(double *thrDweight, double scalingFactor, double *scorePad)
-{
-	if (numSpecific == 0) { // use template to handle this branch at compile time? TODO
-		for (int qx=0; qx < totalQuadPoints; ++qx) {
-			mapDenseSpace(thrDweight[qx], &wherePrep[qx * maxDims],
-				      &whereGram.coeffRef(0, qx), scorePad);
-		}
-	} else {
-		int qloc=0;
-		for (int qx=0; qx < totalQuadPoints; qx++) {
-			const double *whPrep = &wherePrep[qx * maxDims];
-			const double *whGram = &whereGram.coeffRef(0, qx);
-			mapDenseSpace(thrDweight[qloc], whPrep, whGram, scorePad);
-			for (int Sgroup=0; Sgroup < numSpecific; Sgroup++) {
-				mapSpecificSpace(Sgroup, thrDweight[qloc], whPrep, whGram, scorePad);
-				++qloc;
-			}
-		}
-	}
-
-	const int padSize = maxAbilities + triangleLoc1(maxAbilities);
-	for (int d1=0; d1 < padSize; d1++) {
-		scorePad[d1] *= scalingFactor;
-	}
-
-	int cx = maxAbilities;
-	for (int a1=0; a1 < primaryDims; ++a1) {
-		for (int a2=0; a2 <= a1; ++a2) {
-			double ma1 = scorePad[a1];
-			double ma2 = scorePad[a2];
-			scorePad[cx] -= ma1 * ma2;
-			++cx;
-		}
-	}
-	for (int sx=0; sx < numSpecific; sx++) {
-		int sdim = primaryDims + sx;
-		double ma1 = scorePad[sdim];
-		scorePad[maxAbilities + triangleLoc0(sdim)] -= ma1 * ma1;
-	}
-}
-
 ifaGroup::ifaGroup(int cores, bool _twotier) : Rdata(NULL),
 		numThreads(cores), qwidth(6.0), qpoints(49),
 		twotier(_twotier),
@@ -289,7 +217,7 @@ ifaGroup::ifaGroup(int cores, bool _twotier) : Rdata(NULL),
 		mean(0),
 					       cov(0), dataRowNames(0),
 	    weightColumnName(0), rowWeight(0),
-					       minItemsPerScore(1),
+					       minItemsPerScore(NA_INTEGER),
 					       outcomeProb(0), excludedPatterns(-1)
 {}
 
@@ -379,6 +307,11 @@ void ifaGroup::learnMaxAbilities()
 		}
 	}
 	maxAbilities = (loadings != 0).count();
+	if (maxItemDims != maxAbilities) {
+		for (int lx=0; lx < maxItemDims; ++lx) {
+			if (loadings[lx] == 0) Rf_error("Factor %d does not load on any items", 1+lx);
+		}
+	}
 }
 
 void ifaGroup::import(SEXP Rlist)
@@ -646,168 +579,55 @@ void ifaGroup::setMinItemsPerScore(int mips)
 	minItemsPerScore = mips;
 }
 
+void ifaGroup::buildRowSkip()
+{
+	rowSkip.assign(rowMap.size(), false);
+
+	if (maxAbilities == 0) return;
+
+	// Rows with no information about an ability will obtain the
+	// prior distribution as an ability estimate. This will
+	// throw off multigroup latent distribution estimates.
+	for (size_t rx=0; rx < rowMap.size(); rx++) {
+		bool hasNA = false;
+		std::vector<int> contribution(maxAbilities);
+		for (int ix=0; ix < numItems(); ix++) {
+			int pick = dataColumn(ix)[ rowMap[rx] ];
+			if (pick == NA_INTEGER) {
+				hasNA = true;
+				continue;
+			}
+			const double *ispec = spec[ix];
+			int dims = ispec[RPF_ISpecDims];
+			double *iparam = getItemParam(ix);
+			for (int dx=0; dx < dims; dx++) {
+				// assume factor loadings are the first item parameters
+				if (iparam[dx] == 0) continue;
+				contribution[dx] += 1;
+			}
+		}
+		if (!hasNA) continue;
+		if (minItemsPerScore == NA_INTEGER) {
+			Rf_error("You have missing data. You must set minItemsPerScore");
+		}
+		for (int ax=0; ax < maxAbilities; ++ax) {
+			if (contribution[ax] < minItemsPerScore) {
+				// We could compute the other scores, but estimation of the
+				// latent distribution is in the hot code path. We can reconsider
+				// this choice when we try generating scores instead of the
+				// score distribution.
+				rowSkip[rx] = true;
+			}
+		}
+	}
+}
+
 void ifaGroup::sanityCheck() // remove TODO
 {
 }
 
-// Depends on item parameters, but not latent distribution
-void ifaGroup::ba81OutcomeProb(double *param, bool wantLog)
-{
-	const int maxDims = quad.maxDims;
-	outcomeProb = Realloc(outcomeProb, totalOutcomes * quad.totalQuadPoints, double);
-
-#pragma omp parallel for num_threads(numThreads)
-	for (int ix=0; ix < numItems(); ix++) {
-		double *qProb = outcomeProb + cumItemOutcomes[ix] * quad.totalQuadPoints;
-		const double *ispec = spec[ix];
-		int id = ispec[RPF_ISpecID];
-		int dims = ispec[RPF_ISpecDims];
-		Eigen::VectorXd ptheta(dims);
-		double *iparam = param + paramRows * ix;
-		rpf_prob_t prob_fn = wantLog? librpf_model[id].logprob : librpf_model[id].prob;
-
-		for (int qx=0; qx < quad.totalQuadPoints; qx++) {
-			double *where = quad.wherePrep.data() + qx * maxDims;
-			for (int dx=0; dx < dims; dx++) {
-				ptheta[dx] = where[std::min(dx, maxDims-1)];
-			}
-
-			(*prob_fn)(ispec, iparam, ptheta.data(), qProb);
-			qProb += itemOutcomes[ix];
-		}
-	}
-}
-
-void ifaGroup::ba81LikelihoodSlow2(const int px, double *out)
-{
-	const int totalQuadPoints = quad.totalQuadPoints;
-	double *oProb = outcomeProb;
-	std::vector<double> &priQarea = quad.priQarea;
-
-	for (int qx=0; qx < totalQuadPoints; ++qx) {
-		out[qx] = priQarea[qx];
-	}
-
-	const int row = rowMap[px];
-	for (int ix=0; ix < numItems(); ix++) {
-		int pick = dataColumns[ix][row];
-		if (pick == NA_INTEGER) {
-			oProb += itemOutcomes[ix] * totalQuadPoints;
-			continue;
-		}
-		pick -= 1;
-
-		for (int qx=0; qx < totalQuadPoints; ++qx) {
-			out[qx] *= oProb[pick];
-			oProb += itemOutcomes[ix];
-		}
-	}
-}
-
-void ifaGroup::cai2010EiEis(const int px, double *lxk, double *Eis, double *Ei)
-{
-	double *oProb = outcomeProb;
-	const int totalQuadPoints = quad.totalQuadPoints;
-	const int totalPrimaryPoints = quad.totalPrimaryPoints;
-	const int specificPoints = quad.quadGridSize;
-	std::vector<double> &speQarea = quad.speQarea;
-	std::vector<double> &priQarea = quad.priQarea;
-
-	for (int qx=0, qloc = 0; qx < totalPrimaryPoints; qx++) {
-		for (int sx=0; sx < specificPoints * numSpecific; sx++) {
-			lxk[qloc] = speQarea[sx];
-			++qloc;
-		}
-	}
-
-	const int row = rowMap[px];
-	for (int ix=0; ix < numItems(); ix++) {
-		int pick = dataColumns[ix][row];
-		if (pick == NA_INTEGER) {
-			oProb += itemOutcomes[ix] * totalQuadPoints;
-			continue;
-		}
-		pick -= 1;
-		int Sgroup1 = Sgroup[ix];
-		double *out1 = lxk;
-		for (int qx=0; qx < quad.totalQuadPoints; qx++) {
-			out1[Sgroup1] *= oProb[pick];
-			oProb += itemOutcomes[ix];
-			out1 += numSpecific;
-		}
-	}
-
-	for (int qx=0; qx < totalPrimaryPoints * numSpecific; ++qx) Eis[qx] = 0;
-	for (int qx=0; qx < totalPrimaryPoints; ++qx) Ei[qx] = priQarea[qx];
-
-	int eisloc = 0;
-	for (int qx=0, qloc = 0; qx < totalPrimaryPoints; qx++) {
-		for (int sx=0; sx < specificPoints; sx++) {
-			for (int sgroup=0; sgroup < numSpecific; ++sgroup) {
-				double piece = lxk[qloc];
-				Eis[eisloc + sgroup] += piece;
-				++qloc;
-			}
-		}
-		for (int sgroup=0; sgroup < numSpecific; ++sgroup) {
-			Ei[qx] *= Eis[eisloc + sgroup] * quad.getReciprocalOfOne();
-		}
-		eisloc += numSpecific;
-	}
-}
-
-void ifaGroup::cai2010part2(double *Qweight, double *Eis, double *Ei)
-{
-	const int totalPrimaryPoints = quad.totalPrimaryPoints;
-	const int specificPoints = quad.quadGridSize;
-
-	for (int qx=0, qloc = 0; qx < totalPrimaryPoints; qx++) {
-		for (int sgroup=0; sgroup < numSpecific; ++sgroup) {
-			Eis[qloc] = Ei[qx] / Eis[qloc];
-			++qloc;
-		}
-	}
-
-	for (int qloc=0, eisloc=0; eisloc < totalPrimaryPoints * numSpecific; eisloc += numSpecific) {
-		for (int sx=0; sx < specificPoints; sx++) {
-			for (int Sgroup=0; Sgroup < numSpecific; Sgroup++) {
-				Qweight[qloc] *= Eis[eisloc + Sgroup];
-				++qloc;
-			}
-		}
-	}
-}
-
 void ifaGroup::setGridFineness(double width, int points)
 {
 	if (std::isfinite(width)) qwidth = width;
 	if (points != NA_INTEGER) qpoints = points;
 }
-
-double BA81EngineBase::getPatLik(class ifaGroup *state, int px, double *lxk)
-{
-	const int pts = getPrimaryPoints(state);
-	Eigen::ArrayXd &patternLik = state->patternLik;
-	double patternLik1 = 0;
-
-	for (int qx=0; qx < pts; qx++) {
-		patternLik1 += lxk[qx];
-	}
-
-	// This uses the previous iteration's latent distribution.
-	// If we recompute patternLikelihood to get the current
-	// iteration's expected scores then it speeds up convergence.
-	// However, recomputing patternLikelihood and dependent
-	// math takes much longer than simply using the data
-	// we have available here. This is even more true for the
-	// two-tier model.
-	if (!ifaGroup::validPatternLik(patternLik1)) {
-#pragma omp atomic
-		state->excludedPatterns += 1;
-		patternLik[px] = 0;
-		return 0;
-	}
-
-	patternLik[px] = patternLik1;
-	return patternLik1;
-}
